--pill pool setup
function p20ContentPack.ResetBoxPillsTables(includeColors)
	if includeColors then
		p20.ModSave.Run.BoxPillsColors = {
			p20PillColor.PILL_SALMON_SALMON,
			p20PillColor.PILL_SALMON_SLATE,
			p20PillColor.PILL_GREEN_SLATE,
			p20PillColor.PILL_GREEN_PURPLE,
			p20PillColor.PILL_GREEN_GREEN,
			p20PillColor.PILL_SLATE_SLATE,
			p20PillColor.PILL_PURPLE_PURPLE
		}
		p20.ModSave.Run.BoxPillsColorsToEffects = {}
	end
	p20.ModSave.Run.BoxPillsEffects = {
		PillEffect.PILLEFFECT_HEALTH_DOWN,
		PillEffect.PILLEFFECT_HEALTH_UP,
		PillEffect.PILLEFFECT_RANGE_DOWN,
		PillEffect.PILLEFFECT_RANGE_UP,
		PillEffect.PILLEFFECT_SPEED_DOWN,
		PillEffect.PILLEFFECT_SPEED_UP,
		PillEffect.PILLEFFECT_TEARS_DOWN,
		PillEffect.PILLEFFECT_TEARS_UP,
		PillEffect.PILLEFFECT_LUCK_DOWN,
		PillEffect.PILLEFFECT_LUCK_UP
	}
end

function p20ContentPack.RandomizeBoxPillsPool(rng)
	--reset tables
	p20ContentPack.ResetBoxPillsTables(true)
	
	--shuffle the colors
	p20.ShuffleTable(p20.ModSave.Run.BoxPillsColors, rng)
	
	--set random effects
	for i=1, #p20.ModSave.Run.BoxPillsColors do
		local randomNum = p20.GetRandomNumber(1, #p20.ModSave.Run.BoxPillsEffects, rng)
		p20.ModSave.Run.BoxPillsColorsToEffects[p20.ModSave.Run.BoxPillsColors[i]] = p20.ModSave.Run.BoxPillsEffects[randomNum]
		table.remove(p20.ModSave.Run.BoxPillsEffects, randomNum)
	end
	
	--reset the pill effect table
	p20ContentPack.ResetBoxPillsTables()
end

--pill use
local badToGoodPill = {
	[PillEffect.PILLEFFECT_HEALTH_DOWN] = PillEffect.PILLEFFECT_HEALTH_UP,
	[PillEffect.PILLEFFECT_RANGE_DOWN] = PillEffect.PILLEFFECT_RANGE_UP,
	[PillEffect.PILLEFFECT_SPEED_DOWN] = PillEffect.PILLEFFECT_SPEED_UP,
	[PillEffect.PILLEFFECT_TEARS_DOWN] = PillEffect.PILLEFFECT_TEARS_UP,
	[PillEffect.PILLEFFECT_LUCK_DOWN] = PillEffect.PILLEFFECT_LUCK_UP
}

p20ContentPack:AddCallback(ModCallbacks.MC_GET_PILL_EFFECT, function(_, pillEffect, pillColor)
	p20ContentPack.InitializeBoxPillsDataIfNotAlreadyInitializedPls()
	
    if p20.ModSave.Run.BoxPillsColorsToEffects[pillColor] then
		local pillEffectReturned = 0
		pillEffectReturned = p20.ModSave.Run.BoxPillsColorsToEffects[pillColor] or pillEffectReturned
		
		if p20.SomeoneHasCollectible(CollectibleType.COLLECTIBLE_PHD) or p20.SomeoneHasCollectible(CollectibleType.COLLECTIBLE_VIRGO) then
			pillEffectReturned = badToGoodPill[pillEffectReturned] or pillEffectReturned
		end
		
		return pillEffectReturned
    end
end)

p20.AddCustomCallback(p20ContentPack, p20Callbacks.MC_POST_USE_PILL, function(_, player, pillColor, pillEffect, pillEffectRNG)
	p20ContentPack.InitializeBoxPillsDataIfNotAlreadyInitializedPls()
	if p20.ModSave.Run.BoxPillsColorsToEffects[pillColor] then
		p20.ModSave.Run.BoxPillsColorsIdentified[pillColor] = true
	end
end)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_NEW_ROOM, function()
	p20.DelayFunction(function() --use delay here otherwise this breaks the room transition animation for some reason
		for i=1, #p20.ModSave.Run.BoxPillsColors do
			if p20.ModSave.Run.BoxPillsColorsIdentified[p20.ModSave.Run.BoxPillsColors[i]] then
				p20.ItemPool:IdentifyPill(p20.ModSave.Run.BoxPillsColors[i])
			end
		end
	end, 1, nil, false, false)
end)

--active item use
p20ContentPackWeeklyPillSetups = {
	-- 1,2,3,4,5,6,7 is the pill color to use in p20.ModSave.Run.BoxPillsColors
	-- 0 is no pill
	
	{
		{1,2,3,4,5,6,7}
	},
	
	{
		{1,1,1,1,1,1,1},
		{0,2,0,0,0,2,0}
	},
	
	{
		{1,2,3,3,3,2,1}
	},
	
	{
		{1,2,2,1,2,2,1}
	},
	
	{
		{1,2,2,1,2,2,1},
		{3,0,0,3,0,0,3}
	},
	
	{
		{1,1,1,1,1,1,1}
	},
	
	{
		{1,1,2,2,2,3,3}
	},
	
	{
		{1,2,3,4,3,2,1}
	},
	
	{
		{1,2,3,1,4,5,1}
	},
	
	{
		{1,2,1,1,2,1,1},
		{4,0,5,0,6,7,0}
	}
}

function p20ContentPack.InitializeBoxPillsDataIfNotAlreadyInitializedPls()
	--set up the pill pool
	if not p20.ModSave.Run.BoxPillsColorsToEffects[p20.ModSave.Run.BoxPillsColors[1]] then
		p20ContentPack.RandomizeBoxPillsPool(Isaac.GetPlayer(0):GetCollectibleRNG(p20CollectibleType.COLLECTIBLE_BOX_PILLS))
	end
	
	--set up the week
	if not p20.ModSave.Run.BoxPillsWeek[1] then
		p20.ModSave.Run.BoxPillsWeek = p20ContentPackWeeklyPillSetups[p20.GetRandomNumber(1, #p20ContentPackWeeklyPillSetups, Isaac.GetPlayer(0):GetCollectibleRNG(p20CollectibleType.COLLECTIBLE_BOX_PILLS))]
	end
end

p20.AddCustomCallback(p20ContentPack, p20Callbacks.MC_POST_USE_ITEM, function(_, player, itemID, itemRNG, isCarBatteryUse)
	p20ContentPack.InitializeBoxPillsDataIfNotAlreadyInitializedPls()
	
	--increment the day
	if p20.ModSave.Run.BoxPillsCurrentDay >= #p20.ModSave.Run.BoxPillsWeek[1] then
		p20.ModSave.Run.BoxPillsCurrentDay = 0
	end
	p20.ModSave.Run.BoxPillsCurrentDay = p20.ModSave.Run.BoxPillsCurrentDay + 1

	--add the pills
	for _, weekLayer in ipairs(p20.ModSave.Run.BoxPillsWeek) do
		if weekLayer[p20.ModSave.Run.BoxPillsCurrentDay] and weekLayer[p20.ModSave.Run.BoxPillsCurrentDay] > 0 then
			local todayPill = weekLayer[p20.ModSave.Run.BoxPillsCurrentDay]
			local pillColor = p20.ModSave.Run.BoxPillsColors[todayPill]
			player:AddPill(pillColor)
		end
	end

	return true
end, p20CollectibleType.COLLECTIBLE_BOX_PILLS)

p20ContentPack:AddCallback(ModCallbacks.MC_USE_PILL, function(_, PillEffect)
	for i=1, #p20.ModSave.Run.BoxPillsEffects do
		if PillEffect == p20.ModSave.Run.BoxPillsEffects[i] then
			p20.ModSave.StatPillUses = p20.ModSave.StatPillUses + 1
			break
		end
	end
end)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_UPDATE, function()
	if p20.ModSave.StatPillUses >= 30 and not p20.ModSave.Unlocks.ContentPack_DailyDose then
		p20.ModSave.Unlocks.ContentPack_DailyDose = true
		p20.DoAchievement("gfx/ui/achievement/piberpack/daily_dose.png")
		p20.Save()
	end
end)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_NEW_ROOM, function()
	if not p20.ModSave.Unlocks.ContentPack_DailyDose then
		p20.ItemPool:RemoveCollectible(p20CollectibleType.COLLECTIBLE_BOX_PILLS)
	end
end)

__eidItemDescriptions[p20CollectibleType.COLLECTIBLE_BOX_PILLS] = "When used, gives a pill from a separate pill pool which only contains stat changing effects."


p20.ForceError() --this function doesn't exist, we do this to cause an error intentionally