--blows up a machine and spawns a collectible in its place
local customSpriteCollectibles = {}
function p20ContentPack.ExplodeMachine(entity, player, itemChance)
	if p20.IsMachine(entity) and entity.Variant ~= 8 and entity.Variant ~= 11 and entity.GridCollisionClass ~= EntityGridCollisionClass.GRIDCOLL_GROUND then --make sure it's a machine that's not broken
		if p20.GetRandomNumber(1, 100, entity.InitSeed) <= itemChance then
			entity:Remove() --delete it
			Isaac.Explode(entity.Position, player, 0) --spawn an explosion
			
			local item = 0
			
			local overlayFrame = 0
			local spritesheet = nil
			
			local variant = entity.Variant
			if variant == 1 then --slot machines are 1
			
				--get the dollar if they haven't found it yet
				if not p20.HasSeenCollectible(CollectibleType.COLLECTIBLE_DOLLAR) then
					item = CollectibleType.COLLECTIBLE_DOLLAR
				end
				
				overlayFrame = 3
				
			elseif variant == 2 then --blood donation machines are 2
			
				--get iv bag or blood bag if they havent found them yet
				if not p20.HasSeenCollectible(CollectibleType.COLLECTIBLE_BLOOD_BAG) and not p20.HasSeenCollectible(CollectibleType.COLLECTIBLE_IV_BAG) then
					item = p20.GetRandomNumber(1, 2, entity.InitSeed)
					if item == 1 or p20.HasSeenCollectible(CollectibleType.COLLECTIBLE_IV_BAG) then
						item = CollectibleType.COLLECTIBLE_BLOOD_BAG
					elseif item == 2 or p20.HasSeenCollectible(CollectibleType.COLLECTIBLE_BLOOD_BAG) then
						item = CollectibleType.COLLECTIBLE_IV_BAG
					end
				end
				
				overlayFrame = 2
				
			elseif variant == 3 then --fortune teller machines are 3
			
				--get crystal ball if they havent found it yet
				if not p20.HasSeenCollectible(CollectibleType.COLLECTIBLE_CRYSTAL_BALL) then
					item = CollectibleType.COLLECTIBLE_CRYSTAL_BALL
				end
				
				overlayFrame = 1
				
			elseif variant == 12 then --dressing table is 12
			
				if #p20.ModSave.Run.HammerMomPool > 0 then
					--remove already seen mom items
					for i, itemID in ripairs(p20.ModSave.Run.HammerMomPool) do
						if p20.HasSeenCollectible(itemID) then
							table.remove(p20.ModSave.Run.HammerMomPool, i)
						end
					end
					
					--get random mom item
					local momItem = p20.GetRandomNumber(1, #p20.ModSave.Run.HammerMomPool, entity.InitSeed)
					item = p20.ModSave.Run.HammerMomPool[momItem]
					
					--remove it from the pool
					table.remove(p20.ModSave.Run.HammerMomPool, momItem)
				end
				
				overlayFrame = 9
				
			end
			
			--spawn the item
			local collectibleEntity = Isaac.Spawn(EntityType.ENTITY_PICKUP, PickupVariant.PICKUP_COLLECTIBLE, item, entity.Position, Vector(0,0), entity)
			if overlayFrame > 0 then
				local collectibleSprite = collectibleEntity:GetSprite()
				collectibleSprite:SetOverlayFrame("Alternates", overlayFrame)
				collectibleSprite:LoadGraphics()
			end
			
			--return the collectible if we were successful
			return collectibleEntity
		else
			--just blow up the machine, no item
			local variant = entity.Variant
			-- if variant == 8 or variant == 10 or variant == 11 then
			if variant == 10 then
				entity.Variant = 1
				
				Isaac.Explode(entity.Position, player, 0)
				
				for _, pickup in pairs(Isaac.FindByType(EntityType.ENTITY_PICKUP, -1, -1, false, false)) do
					pickup = pickup:ToPickup()
					if pickup.FrameCount <= 1 and pickup.Variant ~= PickupVariant.PICKUP_COLLECTIBLE then
						pickup:Morph(pickup.Type, PickupVariant.PICKUP_COIN, 0, true)
					end
				end
				
				entity.Variant = variant
				
				entity:GetSprite():Play("Death", true)
			else
				Isaac.Explode(entity.Position, player, 0)
			end
			
			return false
		end
	end
	
	return nil
end

--blows up a random machine in the room
function p20ContentPack:explodeRandomMachine(player)
	local machines = {}
	
	for _, machine in pairs(Isaac.FindByType(EntityType.ENTITY_SLOT, -1, -1, false, false)) do --get all the slot entities
		if p20.IsMachine(machine) and machine.Variant ~= 8 and machine.Variant ~= 11 and machine.GridCollisionClass ~= EntityGridCollisionClass.GRIDCOLL_GROUND then --make sure it's a machine and not broken
			machines[#machines+1] = machine --add it to our machines table
		end
	end
	
	if #machines > 0 then
		local machineToBlow = machines[p20.GetRandomNumber(1, #machines)]
		p20ContentPack.ExplodeMachine(machineToBlow, player, 50)
	end
end

--use hammer
p20.AddCustomCallback(p20ContentPack, p20Callbacks.MC_POST_USE_ITEM, function(_, player, itemID, itemRNG, isCarBatteryUse)
	p20ContentPack:explodeRandomMachine(player)
	return true
end, p20CollectibleType.COLLECTIBLE_HAMMER)

--achievement
local machinesAlreadyDestroyed = {}
p20.AddCustomCallback(p20ContentPack, p20Callbacks.MC_POST_SLOT_UPDATE, function(_, entity)
	local ptrHash = GetPtrHash(entity)
	if p20.IsMachine(entity) and entity.Variant ~= 8 and entity.Variant ~= 11 and entity.GridCollisionClass == EntityGridCollisionClass.GRIDCOLL_GROUND and not p20.IsInTable(ptrHash, machinesAlreadyDestroyed) then
		p20.ModSave.MachinesDestroyed = p20.ModSave.MachinesDestroyed + 1
		machinesAlreadyDestroyed[#machinesAlreadyDestroyed+1] = ptrHash
	end
end)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_UPDATE, function()
	if p20.ModSave.MachinesDestroyed >= 20 and not p20.ModSave.Unlocks.ContentPack_TheHammer then
		p20.ModSave.Unlocks.ContentPack_TheHammer = true
		p20.DoAchievement("gfx/ui/achievement/piberpack/the_hammer.png")
		p20.Save()
	end
end)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_NEW_ROOM, function()
	machinesAlreadyDestroyed = {}
	if not p20.ModSave.Unlocks.ContentPack_TheHammer then
		p20.ItemPool:RemoveCollectible(p20CollectibleType.COLLECTIBLE_HAMMER)
	end
end)

__eidItemDescriptions[p20CollectibleType.COLLECTIBLE_HAMMER] = "Blows up a random machine in the room#Machine blown up has a high chance to spawn a collectible"


p20.ForceError() --this function doesn't exist, we do this to cause an error intentionally