function p20ContentPack.CanPlayerPickHolyHearts(player)
	local visibleHearts = p20.GetPlayerVisibleHearts(player)
	
	if player:HasCollectible(p20CollectibleType.COLLECTIBLE_DIVINE_SHROUD) then
		visibleHearts = player:GetHeartLimit()
	end
	
	local mantleEffects = player:GetEffects():GetCollectibleEffectNum(CollectibleType.COLLECTIBLE_HOLY_MANTLE)
	if mantleEffects < visibleHearts then
		return true
	end
	
	return false
end

local holyHeartData = p20.GetPickupData(PickupVariant.PICKUP_HEART, HeartSubType.HEART_SOUL)
holyHeartData.CollectSound = nil
holyHeartData.AddSoulHearts = 0
holyHeartData.AddFliesAsKeeper = 5
holyHeartData.PlayerCollectChecksFunction = function(pickup, player, canPlayerCollect)
	return p20ContentPack.CanPlayerPickHolyHearts(player)
end
holyHeartData.PlayerCollectFunction = function(pickup, player)
	local visibleHearts = p20.GetPlayerVisibleHearts(player)
	
	if player:HasCollectible(p20CollectibleType.COLLECTIBLE_DIVINE_SHROUD) then
		visibleHearts = player:GetHeartLimit()
		player:AddSoulHearts(2)
	end
	
	p20.AddPlayerPersistentMantleEffects(player, 1, visibleHearts, false)
	p20.SFX:Play(SoundEffect.SOUND_HOLY, 1, 0, false, 1.25)
	
	p20.ModSave.Run.HolyHeartsCollected = p20.ModSave.Run.HolyHeartsCollected + 1
end
p20.SetPickupData(p20PickupVariant.PICKUP_HEART_ALT, p20HeartAltSubType.HEART_HOLY, holyHeartData)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_PICKUP_INIT, function(_, pickup)
	if pickup.SubType == p20HeartAltSubType.HEART_HOLY and not p20.ModSave.Unlocks.ContentPack_HolyHearts then
		pickup:Morph(EntityType.ENTITY_PICKUP, PickupVariant.PICKUP_HEART, HeartSubType.HEART_ETERNAL, true)
	end
end, p20PickupVariant.PICKUP_HEART_ALT)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_PICKUP_INIT, function(_, pickup)
	if p20.IsPickupNaturallySpawned(pickup) and p20.ModSave.Unlocks.ContentPack_HolyHearts and pickup.SubType ~= HeartSubType.HEART_ETERNAL and pickup.SubType ~= HeartSubType.HEART_GOLDEN and pickup.SubType ~= HeartSubType.HEART_BONE then
		local rng = RNG()
		rng:SetSeed(pickup.InitSeed, 8)
		
		local holyHeartChance = 0.2
		
		if pickup.SubType == HeartSubType.HEART_SOUL or pickup.SubType == HeartSubType.HEART_HALF_SOUL or pickup.SubType == HeartSubType.HEART_BLACK or p20.SomeoneHasTrinket(TrinketType.TRINKET_WOODEN_CROSS) then
			holyHeartChance = holyHeartChance * 2
		end
		if p20.SomeoneHasCollectible(p20CollectibleType.COLLECTIBLE_DIVINE_SHROUD) then
			holyHeartChance = holyHeartChance * 3
		end
		if p20.SomeoneHasCollectible(CollectibleType.COLLECTIBLE_HOLY_WATER) then
			holyHeartChance = holyHeartChance * 1.5
		end
		if p20.SomeoneHasCollectible(CollectibleType.COLLECTIBLE_HOLY_GRAIL) then
			holyHeartChance = holyHeartChance * 1.3
		end
		
		if holyHeartChance > 10 then
			holyHeartChance = 10
		end
		
		if p20.GetRandomNumber(1, 1000, rng) <= math.floor(holyHeartChance*10) then
			pickup:Morph(EntityType.ENTITY_PICKUP, p20PickupVariant.PICKUP_HEART_ALT, p20HeartAltSubType.HEART_HOLY, true)
		end
	end
end, PickupVariant.PICKUP_HEART)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_PLAYER_UPDATE, function(_, player)
	if player:GetEffects():GetCollectibleEffectNum(CollectibleType.COLLECTIBLE_HOLY_MANTLE) >= 2 and not p20.ModSave.Unlocks.ContentPack_HolyHearts then
		p20.ModSave.Unlocks.ContentPack_HolyHearts = true
		p20.DoAchievement("gfx/ui/achievement/piberpack/holy_hearts.png")
		p20.Save()
	end
end)


p20.ForceError() --this function doesn't exist, we do this to cause an error intentionally