local leakyBatteryData = p20.GetPickupData(PickupVariant.PICKUP_LIL_BATTERY, 0)
leakyBatteryData.FullCharge = false
leakyBatteryData.PlayerCollectChecksFunction = function(pickup, player, canPlayerCollect)
	local data = p20ContentPack.GetData(pickup)
	
	local toReturn = false
	if data.Init then
		local removeBattery = false
		if data.LeakyCanOvercharge then
			if not player:HasCollectible(CollectibleType.COLLECTIBLE_BATTERY) then
				player:AddCollectible(CollectibleType.COLLECTIBLE_BATTERY, 0, false)
				removeBattery = true
			end
		end
		
		if player:NeedsCharge() then
			toReturn = true
		end
		
		if removeBattery then
			player:RemoveCollectible(CollectibleType.COLLECTIBLE_BATTERY)
		end
	end
	
	return toReturn
end
leakyBatteryData.PlayerCollectFunction = function(pickup, player)
	local data = p20ContentPack.GetData(pickup)
	
	if data.Init then
		local secondItem = player.SecondaryActiveItem
		local removeBattery = false
		if data.LeakyCanOvercharge then
			if not player:HasCollectible(CollectibleType.COLLECTIBLE_BATTERY) then
				player:AddCollectible(CollectibleType.COLLECTIBLE_BATTERY, 0, false)
				removeBattery = true
			end
		end
		
		if player:NeedsCharge() then
			local currentCharge = player:GetActiveCharge()
			local maxCharge = p20.ItemConfig:GetCollectible(player:GetActiveItem()).MaxCharges
			
			local chargeToAdd = 1
			if data.LeakyCanOvercharge and currentCharge > maxCharge then
				chargeToAdd = math.ceil(maxCharge * (data.LeakyOverCharge * 0.01))
			else
				chargeToAdd = math.ceil(maxCharge * (data.LeakyCharge * 0.01))
			end
			if chargeToAdd == 0 then
				chargeToAdd = -1
			end
			
			player:SetActiveCharge(currentCharge + chargeToAdd)
			if data.LeakyCanOvercharge then
				p20.SFX:Play(SoundEffect.SOUND_THUMBSUP, 0.5, 0, false, 1.0)
			end
		end
		
		if removeBattery then
			player:RemoveCollectible(CollectibleType.COLLECTIBLE_BATTERY)
		end
		player.SecondaryActiveItem = secondItem
	end
end
p20.SetPickupData(p20PickupVariant.PICKUP_BATTERY_ALT, p20BatteryAltSubType.BATTERY_LEAKY, leakyBatteryData)

local deadBatteryData = p20.GetPickupData(PickupVariant.PICKUP_LIL_BATTERY, 0)
deadBatteryData.FullCharge = false
deadBatteryData.AddCharge = 1
deadBatteryData.PlayerCollectChecksFunction = function(pickup, player, canPlayerCollect)
	if canPlayerCollect and player:HasCollectible(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) then
		return true
	end
	return false
end
p20.SetPickupData(p20PickupVariant.PICKUP_BATTERY_ALT, p20BatteryAltSubType.BATTERY_DEAD, deadBatteryData)

function p20ContentPack:RandomizeLeakyBattery(pickup, data)
	data.LeakyCharge = p20.GetRandomNumber(-5, 100, data.LeakyRNG)
	data.LeakyOverCharge = 0
	data.LeakyCanOvercharge = false
	if p20.GetRandomNumber(1, 30, data.LeakyRNG) == 1 then
		data.LeakyOverCharge = p20.GetRandomNumber(5, 30, data.LeakyRNG)
		data.LeakyCanOvercharge = true
	end
	data.LeakyDamage = 0
	for _, player in pairs(p20.GetPlayersWithCollectible(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER)) do
		local damageToAdd = player.Damage
		for j=1, player:GetCollectibleNum(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) do
			damageToAdd = damageToAdd * 0.5
			data.LeakyDamage = data.LeakyDamage + damageToAdd
		end
	end
	if data.LeakyDamage < 2 then
		data.LeakyDamage = 2
	end
	data.LeakyDamage = data.LeakyDamage * (p20.GetRandomNumber(50, 100, data.LeakyRNG) * 0.01)
end

p20ContentPack:AddCallback(ModCallbacks.MC_POST_PICKUP_UPDATE, function(_, pickup)
	if pickup.SubType == p20BatteryAltSubType.BATTERY_LEAKY then
		local sprite, data = pickup:GetSprite(), p20ContentPack.GetData(pickup)
		
		if not data.Init then
			data.LeakyRNG = p20.GetInitializedRNG()
			data.LeakyRNG:SetSeed(pickup.InitSeed, 5)
			if p20.SomeoneHasCollectible(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) then
				data.LeakyLaserTimer = p20.GetRandomNumber(40, 80, data.LeakyRNG)
			else
				data.LeakyLaserTimer = p20.GetRandomNumber(60, 120, data.LeakyRNG)
			end
			p20ContentPack:RandomizeLeakyBattery(pickup, data)
			data.Init = true
		end
		
		if sprite:IsPlaying("Idle") then
			data.LeakyLaserTimer = data.LeakyLaserTimer - 1
			if data.LeakyLaserTimer <= 0 then
				if p20.SomeoneHasCollectible(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) then
					data.LeakyLaserTimer = p20.GetRandomNumber(40, 80, data.LeakyRNG)
				else
					data.LeakyLaserTimer = p20.GetRandomNumber(60, 120, data.LeakyRNG)
				end
				if p20.GetRandomNumber(1, 3, data.LeakyRNG) == 1 then
					sprite:Play("FireLaserDouble", true)
				else
					sprite:Play("FireLaser", true)
				end
			end
		end
		
		if sprite:IsPlaying("FireLaser") or sprite:IsPlaying("FireLaserDouble") then
			if sprite:IsEventTriggered("FireLaser") then
				p20ContentPack:RandomizeLeakyBattery(pickup, data)
				local angle = nil
				
				local enemiesExist = false
				local hitEnemy = nil
				for _, enemy in pairs(Isaac.FindInRadius(pickup.Position, 120 + (p20.EveryoneHasCollectibleNum(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) * 22), EntityPartition.ENEMY)) do
					if enemy:IsActiveEnemy(false) and enemy:IsVulnerableEnemy() and not (enemy:HasEntityFlags(EntityFlag.FLAG_FRIENDLY) or enemy:HasEntityFlags(EntityFlag.FLAG_NO_TARGET)) then
						enemiesExist = true
						
						local chance = 20
						if p20.SomeoneHasCollectible(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) then
							chance = 80
						end
						if p20.GetRandomNumber(1, 100, data.LeakyRNG) <= chance then
							angle = (enemy.Position - pickup.Position):GetAngleDegrees()
							hitEnemy = enemy
							break
						end
					end
				end
				
				local hitPlayer = nil
				if not enemiesExist then
					for _, player in pairs(Isaac.FindInRadius(pickup.Position, 120 + (p20.EveryoneHasCollectibleNum(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) * 22), EntityPartition.PLAYER)) do
						if not p20.IsPlayerGhost(player) then
							if p20.GetRandomNumber(1, 10, data.LeakyRNG) == 1 then
								angle = (player.Position - pickup.Position):GetAngleDegrees()
								hitPlayer = player
								break
							end
						end
					end
				end
				
				if not angle then
					angle = p20.GetRandomNumber(0, 360, data.LeakyRNG)
				end
				
				local laser = EntityLaser.ShootAngle(2, pickup.Position + Vector(0,-20), angle, 4, Vector(0, -1) + Vector(-1, 0):Rotated(angle), pickup)
				laser.DepthOffset = pickup.DepthOffset + 1
				laser.RenderZOffset = pickup.RenderZOffset + 1
				laser.CollisionDamage = 0
				laser.EntityCollisionClass = EntityCollisionClass.ENTCOLL_NONE
				laser:GetSprite().Color = Color(1,1,1,1,255,200,0)
				
				if hitEnemy then
					laser.MaxDistance = hitEnemy.Position:Distance(pickup.Position)
					hitEnemy:TakeDamage(data.LeakyDamage, DamageFlag.DAMAGE_LASER, EntityRef(pickup), 1)
				elseif hitPlayer then
					laser.MaxDistance = hitPlayer.Position:Distance(pickup.Position)
					hitPlayer:TakeDamage(1, DamageFlag.DAMAGE_FAKE, EntityRef(pickup), 1)
					p20.ModSave.HitsFromLeakyBatteries = p20.ModSave.HitsFromLeakyBatteries + 1
				else
					laser.MaxDistance = (100 + (p20.EveryoneHasCollectibleNum(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) * 20)) * (p20.GetRandomNumber(50, 120, data.LeakyRNG) * 0.01)
				end
				
				local deadChance = 5
				if p20.SomeoneHasCollectible(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) then
					deadChance = 1
				end
				if p20.GetRandomNumber(1, 100, data.LeakyRNG) <= deadChance then
					pickup.SubType = p20BatteryAltSubType.BATTERY_DEAD
					sprite:Load("gfx/items/pick ups/piberpack/batteries/dead_battery.anm2", true)
					sprite:Play("FromLeaky", true)
				end
			end
		end
		if sprite:IsFinished("FireLaser") or sprite:IsFinished("FireLaserDouble") then
			sprite:Play("Idle", true)
		end
	elseif pickup.SubType == p20BatteryAltSubType.BATTERY_DEAD then
		local sprite = pickup:GetSprite()
		if sprite:IsFinished("FromLeaky") then
			sprite:Play("Idle", true)
		end
	end
end, p20PickupVariant.PICKUP_BATTERY_ALT)

p20ContentPack:AddCallback(ModCallbacks.MC_ENTITY_TAKE_DMG, function(_, entity, amount, flags, source, countdown)
	if source.Type == EntityType.ENTITY_PICKUP and source.Variant == p20PickupVariant.PICKUP_BATTERY_ALT and source.Entity.SubType == p20BatteryAltSubType.BATTERY_LEAKY and flags & DamageFlag.DAMAGE_FAKE == 0 then
		return false
	end
end, EntityType.ENTITY_PLAYER)

function p20ContentPack:onBatteryInit(pickup)
	if p20.ModSave.Unlocks.ContentPack_LeakyBatteries and (p20.IsPickupNaturallySpawned(pickup) or (pickup.Variant == p20PickupVariant.PICKUP_BATTERY_ALT and pickup.SubType == 0)) then
		local rng = RNG()
		rng:SetSeed(pickup.InitSeed, 7)
		
		local leakyBatteryChance = 2 + (p20.EveryoneHasCollectibleNum(p20CollectibleType.COLLECTIBLE_BATTERY_TESTER) * 15)
		if p20.GetRandomNumber(1, 100, rng) <= leakyBatteryChance then
			pickup:Morph(EntityType.ENTITY_PICKUP, p20PickupVariant.PICKUP_BATTERY_ALT, p20BatteryAltSubType.BATTERY_LEAKY, true)
			return
		end
	end
end
p20ContentPack:AddCallback(ModCallbacks.MC_POST_PICKUP_INIT, p20ContentPack.onBatteryInit, p20PickupVariant.PICKUP_BATTERY_ALT)
p20ContentPack:AddCallback(ModCallbacks.MC_POST_PICKUP_INIT, p20ContentPack.onBatteryInit, PickupVariant.PICKUP_LIL_BATTERY)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_UPDATE, function()
	if p20.ModSave.BatteriesPickedUp >= 35 and not p20.ModSave.Unlocks.ContentPack_LeakyBatteries then
		p20.ModSave.Unlocks.ContentPack_LeakyBatteries = true
		p20.DoAchievement("gfx/ui/achievement/piberpack/leaky_battery.png")
		p20.Save()
	end
end)


p20.ForceError() --this function doesn't exist, we do this to cause an error intentionally