p20ContentPack:AddCallback(ModCallbacks.MC_POST_NPC_INIT, function(_, npc)
	local replacedData = p20.GetReplacedAtRoomSpawnData(npc)
	if replacedData and replacedData.Type == EntityType.ENTITY_VIS and (replacedData.Variant == p20NPCVariant.SPEWER or replacedData.Variant == p20NPCVariant.SPEWER_DANK) then
		p20.RevertRoomSpawnReplacedNpc(npc)
	end
end, EntityType.ENTITY_VIS)

local dankColor = Color(0.1,0.1,0.1,1,0,0,0)
p20.AddCustomCallback(p20, ModCallbacks.MC_PRE_NPC_UPDATE, function(_, npc)
	if npc.Variant == p20NPCVariant.SPEWER or npc.Variant == p20NPCVariant.SPEWER_DANK then
		if npc.State == 9 then
			local sprite, data = npc:GetSprite(), p20ContentPack.GetData(npc)
			data.BulletsSpewed = data.BulletsSpewed or 0
			
			if not data.SpewingDirection and not sprite:WasEventTriggered("Stop") then
				local target = npc:GetPlayerTarget()
				
				local toTarget = p20.VECTOR_DOWN
				if target then
					toTarget = target.Position - npc.Position
				end
				
				local x, y = toTarget.X, toTarget.Y
				if math.abs(x) > math.abs(y) then
					if x < 0 then -- left
						data.SpewingDirection = p20.VECTOR_LEFT
						sprite.FlipX = true
					else -- right
						data.SpewingDirection = p20.VECTOR_RIGHT
						sprite.FlipX = false
					end
					sprite:Play("Attack02Horiz")
				else
					if y < 0 then -- up
						data.SpewingDirection = p20.VECTOR_UP
						sprite:Play("Attack02Up")
					else -- down
						data.SpewingDirection = p20.VECTOR_DOWN
						sprite:Play("Attack02Down")
					end
				end
			end
			
			npc.Velocity = npc.Velocity * 0.8
			
			if sprite:IsEventTriggered("Shoot") then
				data.BulletsSpewed = 0
				npc:PlaySound(SoundEffect.SOUND_BLOODSHOOT, 0.8, 0, false, 1)
			end
		
			if (sprite:IsEventTriggered("Shoot") or sprite:WasEventTriggered("Shoot"))
			and not (sprite:IsEventTriggered("Stop") or sprite:WasEventTriggered("Stop"))
			and npc.FrameCount%2 == 0 then
			
				local params = ProjectileParams()
				params.FallingAccelModifier = 0.5
				
				if npc.Variant == p20NPCVariant.SPEWER_DANK then
					params.Color = dankColor
				end
				
				local proj = npc:FireProjectiles(npc.Position, (data.SpewingDirection * ((25 - math.max(data.BulletsSpewed,1)) * 0.8)):Rotated(math.random(-50,50)*0.1), 0, params)
				data.BulletsSpewed = data.BulletsSpewed + 1
				
			end
			
			if sprite:IsEventTriggered("Stop") then
				data.BulletsSpewed = 0
				data.SpewingDirection = nil
			end
			
			if sprite:IsFinished("Attack02Horiz") or sprite:IsFinished("Attack02Up") or sprite:IsFinished("Attack02Down") then
				npc.State = 4
			end
			
			return true
		end
	end
end, EntityType.ENTITY_VIS)

p20ContentPack:AddCallback(ModCallbacks.MC_POST_ENTITY_REMOVE, function(_, entity)
	if not p20.Game:IsPaused() and entity.SpawnerType == EntityType.ENTITY_VIS and (entity.SpawnerVariant == p20NPCVariant.SPEWER or entity.SpawnerVariant == p20NPCVariant.SPEWER_DANK) then
		local creepVariant = EffectVariant.CREEP_RED
		if entity.SpawnerVariant == p20NPCVariant.SPEWER_DANK then
			creepVariant = EffectVariant.CREEP_BLACK
		end
		local creep = Isaac.Spawn(EntityType.ENTITY_EFFECT, creepVariant, 0, entity.Position, p20.VECTOR_ZERO, entity):ToEffect()
		local scale = creep.Scale * 1.2
		creep.SpriteScale = Vector(scale, scale)
		creep:Update()
	end
end, EntityType.ENTITY_PROJECTILE)


p20.ForceError() --this function doesn't exist, we do this to cause an error intentionally